@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2016 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -171,7 +171,7 @@ expint <- function (x, deriv = 0) {
       stop("Bad input for argument 'deriv'")
     answer <- rep_len(0, length(x))
     if (deriv == 1) {
-      answer <- exp(x) / x  
+      answer <- exp(x) / x
     }
     if (deriv == 2) {
       answer <- exp(x) / x - exp(x) / x^2
@@ -222,7 +222,7 @@ expint.E1 <- function (x, deriv = 0) {
       stop("Bad input for argument 'deriv'")
     answer <- rep_len(0, length(x))
     if (deriv == 1) {
-      answer <- exp(-x) / x  
+      answer <- exp(-x) / x
     }
     if (deriv == 2) {
       answer <- exp(-x) / x + exp(-x) / x^2
@@ -283,6 +283,13 @@ expexpint <- function(x) {
 }
 
 
+if (FALSE)
+pochhammer <- function (x, n) {
+  exp(lgamma(x+n) - lgamma(x))
+}
+
+
+
 
 
 
@@ -308,4 +315,134 @@ expint.E1 <- function(x) {
 
 
 
+ Zeta.aux <- function(shape, qq, shift = 1) {
+
+
+
+  LLL <- max(length(shape), length(qq))
+  if (length(shape) != LLL) shape <- rep_len(shape, LLL)
+  if (length(qq   ) != LLL) qq    <- rep_len(qq,    LLL)
+
+  if (any(qq < 12-1))
+    warning("all values of argument 'q' should be 12 or more")
+  aa <- qq
+
+
+  B2 <- c(1/6, -1/30, 1/42, -1/30, 5/66, -691/2730, 7/6, -3617/510)
+  kk <- length(B2)  # 8
+  ans <- 1 / ((shape-1) * (shift + aa)^(shape-1)) +
+         0.5 / (shift + aa)^shape
+
+  term <- (shape/2) / (shift + aa)^(shape+1)
+  ans <- ans + term * B2[1]
+
+  for (mm in 2:kk) {
+    term <- term * (shape+2*mm-3) *
+            (shape+2*mm-2) / ((2*mm-1) * 2 * mm * (shift + aa)^2)
+    ans <- ans + term * B2[mm]
+  }
+  ifelse(aa - 1 <= qq, ans, rep(0, length(ans)))  # Handled above
+}
+
+
+
+
+
+
+
+ zeta <- function(x, deriv = 0, shift = 1) {
+
+
+
+
+  deriv.arg <- deriv
+  rm(deriv)
+  if (!is.Numeric(deriv.arg, length.arg = 1,
+                  integer.valued = TRUE))
+    stop("'deriv' must be a single non-negative integer")
+  if (deriv.arg < 0 || deriv.arg > 2)
+    stop("'deriv' must be 0, 1, or 2")
+
+
+  if (deriv.arg > 0)
+    return(Zeta.derivative(x, deriv.arg = deriv.arg, shift = shift))
+
+
+
+  if (any(special <- Re(x) <= 1)) {
+    ans <- x
+    ans[special] <- Inf  # For Re(x) == 1
+
+    special3 <- Re(x) < 1
+    ans[special3] <- NA  # For 0 < Re(x) < 1
+
+    special4 <- (0 < Re(x)) & (Re(x) < 1) & (Im(x) == 0)
+    ans[special4] <-
+      Zeta.derivative(x[special4], deriv.arg = deriv.arg, shift = shift)
+
+
+    special2 <- Re(x) < 0
+    if (any(special2)) {
+      x2 <- x[special2]
+      cx <- 1 - x2
+      ans[special2] <- 2^(x2) * pi^(x2-1) * sin(pi*x2/2) *
+                       gamma(cx) * Recall(cx)
+    }  # special2
+
+    if (any(!special)) {
+      ans[!special] <- Recall(x[!special])
+    }
+    return(ans)
+  }  # special
+
+  aa <- 12
+  ans <- 0
+  for (ii in 0:(aa-1))
+    ans <- ans + 1 / (shift + ii)^x
+
+  ans <- ans + Zeta.aux(shape = x, aa, shift = shift)
+  ans[shift <= 0] <- NaN
+  ans
+}  # zeta
+
+
+
+ Zeta.derivative <- function(x, deriv.arg = 0, shift = 1) {
+
+
+
+  if (!all(shift == 1))
+    stop("currently 'shift' must all be 1")
+
+
+  if (!is.Numeric(deriv.arg, length.arg = 1,
+                  integer.valued = TRUE))
+    stop("'deriv.arg' must be a single non-negative integer")
+  if (deriv.arg < 0 || deriv.arg > 2)
+    stop("'deriv.arg' must be 0, 1, or 2")
+
+  if (any(Im(x) != 0))
+    stop("Sorry, currently can only handle x real, not complex")
+  if (any(x < 0))
+    stop("Sorry, currently cannot handle x < 0")
+
+  ok <- is.finite(x) & x > 0 & x != 1  # Handles NAs
+  ans <- rep_len(NA_real_, length(x))
+  nn <- sum(ok)  # Effective length (excludes x < 0 and x = 1 values)
+  if (nn)
+    ans[ok] <- .C("vzetawr", as.double(x[ok]), ans = double(nn),
+                  as.integer(deriv.arg), as.integer(nn))$ans
+
+
+
+  if (deriv.arg == 0)
+    ans[is.finite(x) & abs(x) < 1.0e-12] <- -0.5
+
+  ans
+}
+
+
+
+
+
 
